import java.awt.Canvas;
import java.awt.Color;
import java.awt.EventQueue;
import java.awt.Font;
import java.awt.Graphics;
import java.awt.event.ComponentAdapter;
import java.awt.event.ComponentEvent;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;

import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JTextField;
import java.awt.Toolkit;

/*        Method of the least squares using a  Cholesky decomposition         */
/*                             www.mosismath.com                              */

public class mainFrame {

	private JFrame frmMethodOfThe;
	private JPanel pGraph; 
	int samples = 7;       // number of reference points
	int order = 3;         // order of the polynomial
    int datapoints = 1000; // number of datapoints for the interpolation
   
    double integral = 0;
    public double deltaX = 0.5;

    double[] yp = new double[datapoints];
    double[] xp = new double[datapoints];
    double[] y_k = new double[samples];
    double[] x_k = new double[samples];
    double[][] m = new double[samples][samples];
    double[] x = new double[samples];
    double[] y = new double[samples];
    double[] d= new double[samples];
    double[][] l = new double[samples][samples];
    
    private JTextField[] tbs = new JTextField[samples];
    private JTextField[] tbx = new JTextField[samples];
    private JTextField textBoxY1;
    private JTextField textBoxY2;
    private JTextField textBoxY3;
    private JTextField textBoxY4;
    private JTextField textBoxY5;
    private JTextField textBoxY6;
    private JTextField textBoxX1;
    private JTextField textBoxX2;
    private JTextField textBoxX3;
    private JTextField textBoxX4;
    private JTextField textBoxX5;
    private JTextField textBoxX6;
    private JTextField textBoxY7;
    private JTextField textBoxX7;
    
    public class TPoint  {
		public float X;
        public float Y;	
	}

    
    public class myCanvas extends Canvas {
    	
		private static final long serialVersionUID = 1L;
		
		public void paint(Graphics g) { 		 			              
			  drawGraph(g, datapoints, x_k[0], x_k[samples - 1], true);
		}
    }
   
    private boolean Cholesky_decomp()
    {
        int i, j, k;
        boolean result = true;

        for (i = 0; i < order; i++)
        {
            for (j = 0; j <= i; j++)
            {
                if (i > j)
                {
                    l[i][j] = m[i][j];
                    for (k = 0; k < j; k++)
                        l[i][j] = l[i][j] - l[i][k] * l[j][k];

                    if (Math.abs(l[j][j]) > 1E-200)
                        l[i][j] = l[i][j] / l[j][j];
                    else
                        result = false;
                }
                if (i == j)
                {
                    l[i][i] = m[i][i];
                    for (k = 0; k < j; k++)
                    {
                        l[i][i] = l[i][i] - (l[i][k] * l[i][k]);
                    }
                    if (l[i][i] >= 0)
                        l[i][i] = Math.sqrt(l[i][i]);
                    else
                        result = false;
                }
            }
        }
        return result;
    }

    boolean Solve()
    {
        boolean result = true;
        int i, j;
        for (j = 0; j < order; j++)
        {
            y[j] = d[j];
            for (i = 0; i < j; i++)
            {
                y[j] = y[j] - y[i] * l[j][i];
            }
            if (Math.abs(l[j][j]) > 1E-200)
                y[j] = y[j] / l[j][j];
            else
                result = false;
        }
        x[order - 1] = y[order - 1] / l[order - 1][order - 1];

        for (j = order - 2; j >= 0; j--)
        {
            x[j] = y[j];
            for (i = order - 1; i > j; i--)
            {
                x[j] = x[j] - x[i] * l[i][j];
            }
            if (Math.abs(l[j][j]) > 1E-200)
                x[j] = x[j] / l[j][j];
            else
                result = false;
        }
        return result;
    }

    
    public void Interpolate(double[] x, int maxPoints, double maxTime)
    {
        int i, k;
        for (i = 0; i < maxPoints; i++)
        {
            yp[i] = 0;
            xp[i] = (double)(i) * maxTime / (double)maxPoints;
            for (k = 0; k < order; k++)
            {   
                yp[i] = yp[i] + x[k] * Math.pow( xp[i], (double)(k));
            }
        }
    }
   
   private void calculate()
   {
	   int i, j, k;
       double[][] ta = new double[samples][samples]; // temp Matrix
       double[] td = new double[samples]; // temp Matrix
    
       if (samples < 10)
       {
           for (i = 0; i < samples; i++)
           {
               if (tbs[i].getText() != "")
               {
            	   y_k[i] = Double.parseDouble(tbs[i].getText());
                   x_k[i] = Double.parseDouble(tbx[i].getText());
               }
               else
                   y_k[i] = 0.0;
               x[i] = 0.0;
           }
       }

       // fill matrix
       for (i = 0; i < samples; i++)
       {
           td[i] = y_k[i];
           for (k = 0; k < order; k++)
           {
               ta[i][k] = Math.pow(x_k[i], k);
           }   
       }

       //calc a * transposed(a) and y * transposed(a)
       for (i = 0; i < order; i++)
       {
           d[i] = 0.0;
           for (k = 0; k < samples; k++)
               d[i] = d[i] + ta[k][i] * td[k];

           for (j = 0; j < order; j++)
           {
               m[j][i] = 0.0;
               for (k = 0; k < samples; k++)
               {
                   m[j][i] = m[j][i] + ta[k][j] * ta[k][i];
               }
           }
       }

       /*solve Matrix equation by Gauss*/
       if (Cholesky_decomp())
       {
           Solve();        
           Interpolate(x, datapoints, x_k[samples - 1]);
       }
       else
    	   JOptionPane.showMessageDialog(frmMethodOfThe, "Matrix calculattion error :-(");
   }

   
   public void drawGraph(Graphics g, int maxPoints, double minX, double maxX, boolean doClear)
   {
       TPoint p1, p2;
       int j;
       double maxTime = -1.0;
       double maxVal = -1.0;
       double minVal = 1.0;
       double scalefactor;
       boolean drawCeroline;
       for (j = 0; j < maxPoints - 1; j++)
       {
           //if ((xp[j] >= minX) && (xp[j] <= maxX))
           {
               if (maxTime < xp[j])
                   maxTime = xp[j];
               if (maxVal < yp[j])
                   maxVal = yp[j];
               if (minVal > yp[j])
                   minVal = yp[j];
           }
       }
       maxTime = maxTime * 1.05;
       maxTime = maxTime * 1.05;
       if (minVal < 0)
       {
           if (maxVal > 0)
           {
               if (maxVal > Math.abs(minVal))
                   scalefactor = pGraph.getHeight() / maxVal / 2.2;
               else
                   scalefactor = pGraph.getHeight() / Math.abs(minVal) / 2.2;
           }
           else
               scalefactor = 5.0;
           drawCeroline = true;
       }
       else
       {
           scalefactor = pGraph.getHeight() / (maxVal) / 1.1;
           drawCeroline = false;
       }
       p1 = new TPoint();
       p2 = new TPoint();
       if (doClear)
       	g.clearRect(0, 0, pGraph.getWidth(), pGraph.getHeight());
      
       g.setColor(new Color(0, 0, 255));
       if (drawCeroline)
       {
           p1.X = 0;
           p1.Y = (pGraph.getHeight() / 2);
           p2.X = pGraph.getWidth();
           p2.Y = (pGraph.getHeight() / 2);
           g.setColor(new Color(0, 0, 0));
           g.drawLine((int)Math.round(p1.X),(int)Math.round(p1.Y),(int)Math.round( p2.X),(int)Math.round(p2.Y)); 
       }
       g.setColor(new Color(0, 0, 255));
       if (maxTime > 0) // draw interpolated graph
       {
           for (j = 0; j < maxPoints - 1; j++)
           {
               if ((xp[j] >= minX) && (xp[j] <= maxX))
               {
                   p1.X = (int)Math.round(xp[j] * pGraph.getWidth() / (double)maxTime);
                   p2.X = (int)Math.round(xp[j + 1] * pGraph.getWidth() / (double)maxTime);
                   if (drawCeroline)
                   {
                       p1.Y = (pGraph.getHeight() / 2) - (int)Math.round(yp[j] * scalefactor);
                       p2.Y = (pGraph.getHeight() / 2) - (int)Math.round(yp[j + 1] * scalefactor);
                   }
                   else
                   {
                       p1.Y = (pGraph.getHeight() - (int)Math.round(yp[j] * scalefactor));
                       p2.Y = (pGraph.getHeight() - (int)Math.round(yp[j + 1] * scalefactor));
                   }
                   g.drawLine((int)Math.round(p1.X),(int)Math.round(p1.Y),(int)Math.round( p2.X),(int)Math.round(p2.Y)); 
               }
           }
           g.setColor(new Color(255, 0, 0));
           for (j = 0; j < samples; j++) // mark reference points
           {
               p1.X = (int)Math.round(x_k[j] * pGraph.getWidth() / (double)maxTime);
               if (drawCeroline)
                   p1.Y = (pGraph.getHeight() / 2) - (int)Math.round(y_k[j] * scalefactor);
               else
                   p1.Y = (pGraph.getHeight() - (int)Math.round(y_k[j] * scalefactor));
               g.drawOval((int)Math.round(p1.X-2), (int)Math.round(p1.Y-2), 4, 4);
           }
       }
   }
   


	/**
	 * Launch the application.
	 */
	public static void main(String[] args) {
		EventQueue.invokeLater(new Runnable() {
			public void run() {
				try {
					mainFrame window = new mainFrame();
					window.frmMethodOfThe.setVisible(true);
				} catch (Exception e) {
					e.printStackTrace();
				}
			}
		});
	}

	/**
	 * Create the application.
	 */
	public mainFrame() {
		int i;
		initialize();
		tbs[0] = textBoxY1;
		tbs[1] = textBoxY2;
		tbs[2] = textBoxY3;
		tbs[3] = textBoxY4;
		tbs[4] = textBoxY5;
		tbs[5] = textBoxY6;
		tbs[6] = textBoxY7;
		tbx[0] = textBoxX1;
		tbx[1] = textBoxX2;
		tbx[2] = textBoxX3;
		tbx[3] = textBoxX4;
		tbx[4] = textBoxX5;
		tbx[5] = textBoxX6;
		tbx[6] = textBoxX7;
		
		
		// some sample values for y
        y_k[0] = 2.63;
        y_k[1] = 1.18;
        y_k[2] = 1.16;
        y_k[3] = 1.54;
        y_k[4] = 2.65;
        y_k[5] = 5.41;
        y_k[6] = 7.67;

        // some sample values for y
        x_k[0] = 0.04;
        x_k[1] = 0.32;
        x_k[2] = 0.51;
        x_k[3] = 0.73;
        x_k[4] = 1.03;
        x_k[5] = 1.42;
        x_k[6] = 1.6;
       
       for (i = 0; i < samples; i++)
       {
    	
           tbs[i].setText(Double.toString(y_k[i]));
           tbx[i].setText(Double.toString(x_k[i]));
       }
	}

	/**
	 * Initialize the contents of the frame.
	 */
	private void initialize() {
		frmMethodOfThe = new JFrame();
		frmMethodOfThe.setTitle("Method of the least squares");
		frmMethodOfThe.setIconImage(Toolkit.getDefaultToolkit().getImage(mainFrame.class.getResource("/Ressource/Icon.png")));
		frmMethodOfThe.setBounds(100, 100, 450, 300);
		frmMethodOfThe.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		frmMethodOfThe.getContentPane().setLayout(null);
		frmMethodOfThe.setBounds(100, 100, 840, 536);
		pGraph = new JPanel();
		final Canvas canvas = new myCanvas();
		canvas.setBackground(Color.WHITE);
		pGraph.addComponentListener(new ComponentAdapter() {
			@Override
			public void componentResized(ComponentEvent arg0) {
				canvas.setBounds(0, 0, pGraph.getWidth(), pGraph.getHeight());
			}
		});
		pGraph.setBackground(Color.WHITE);
		pGraph.setBounds(10, 11, 800, 393);
		frmMethodOfThe.getContentPane().add(pGraph);
		pGraph.add(canvas);
		pGraph.setLayout(null);
		
		textBoxY1 = new JTextField();
		textBoxY1.setColumns(10);
		textBoxY1.setBounds(49, 416, 62, 30);
		frmMethodOfThe.getContentPane().add(textBoxY1);
		
		textBoxY2 = new JTextField();
		textBoxY2.setColumns(10);
		textBoxY2.setBounds(121, 416, 62, 30);
		frmMethodOfThe.getContentPane().add(textBoxY2);
		
		textBoxY3 = new JTextField();
		textBoxY3.setColumns(10);
		textBoxY3.setBounds(193, 416, 62, 30);
		frmMethodOfThe.getContentPane().add(textBoxY3);
		
		textBoxY4 = new JTextField();
		textBoxY4.setColumns(10);
		textBoxY4.setBounds(265, 416, 62, 30);
		frmMethodOfThe.getContentPane().add(textBoxY4);
		
		textBoxY5 = new JTextField();
		textBoxY5.setColumns(10);
		textBoxY5.setBounds(337, 416, 62, 30);
		frmMethodOfThe.getContentPane().add(textBoxY5);
		
		textBoxY6 = new JTextField();
		textBoxY6.setColumns(10);
		textBoxY6.setBounds(409, 416, 62, 30);
		frmMethodOfThe.getContentPane().add(textBoxY6);
		
		textBoxX1 = new JTextField();
		textBoxX1.setColumns(10);
		textBoxX1.setBounds(49, 457, 62, 30);
		frmMethodOfThe.getContentPane().add(textBoxX1);
		
		textBoxX2 = new JTextField();
		textBoxX2.setColumns(10);
		textBoxX2.setBounds(121, 457, 62, 30);
		frmMethodOfThe.getContentPane().add(textBoxX2);
		
		textBoxX3 = new JTextField();
		textBoxX3.setColumns(10);
		textBoxX3.setBounds(193, 457, 62, 30);
		frmMethodOfThe.getContentPane().add(textBoxX3);
		
		textBoxX4 = new JTextField();
		textBoxX4.setColumns(10);
		textBoxX4.setBounds(265, 457, 62, 30);
		frmMethodOfThe.getContentPane().add(textBoxX4);
		
		textBoxX5 = new JTextField();
		textBoxX5.setColumns(10);
		textBoxX5.setBounds(337, 457, 62, 30);
		frmMethodOfThe.getContentPane().add(textBoxX5);
		
		textBoxX6 = new JTextField();
		textBoxX6.setColumns(10);
		textBoxX6.setBounds(409, 457, 62, 30);
		frmMethodOfThe.getContentPane().add(textBoxX6);
		
		textBoxY7 = new JTextField();
		textBoxY7.setColumns(10);
		textBoxY7.setBounds(481, 415, 62, 30);
		frmMethodOfThe.getContentPane().add(textBoxY7);
		
		textBoxX7 = new JTextField();
		textBoxX7.setColumns(10);
		textBoxX7.setBounds(481, 457, 62, 30);
		frmMethodOfThe.getContentPane().add(textBoxX7);
		
		JLabel lblNewLabel = new JLabel("Y");
		lblNewLabel.setFont(new Font("Tahoma", Font.PLAIN, 14));
		lblNewLabel.setBounds(20, 423, 46, 14);
		frmMethodOfThe.getContentPane().add(lblNewLabel);
		
		JLabel lblX = new JLabel("X");
		lblX.setFont(new Font("Tahoma", Font.PLAIN, 14));
		lblX.setBounds(20, 464, 46, 14);
		frmMethodOfThe.getContentPane().add(lblX);
		
		JButton btCalc = new JButton("Calculate");
		btCalc.addMouseListener(new MouseAdapter() {
			@Override
			public void mouseClicked(MouseEvent e) {
				calculate();
				canvas.repaint();
			}
		});
		btCalc.setBounds(698, 415, 112, 72);
		frmMethodOfThe.getContentPane().add(btCalc);
	}
}
